@@ -26,10 +26,7 @@ class DisabledSdist(sdist):
 
 
 def main():
-    try:
-        from setuptools import setup
-    except ImportError:
-        from distutils.core import setup
+    from setuptools import setup
 
     # Make sure the to-be-installed version of Twisted is used, if available,
     # since we're importing from it:
@@ -37,12 +34,17 @@ def main():
         sys.path.insert(0, '.')
 
     from twisted.python.dist3 import modulesToInstall
-    from twisted.python.dist import STATIC_PACKAGE_METADATA
+    from twisted.python.dist3 import testDataFiles, _processDataFileList
+    from twisted.python.dist import STATIC_PACKAGE_METADATA, getDataFiles
 
+    _dataFiles = _processDataFileList(testDataFiles)
     args = STATIC_PACKAGE_METADATA.copy()
     args['install_requires'] = ["zope.interface >= 4.0.2"]
     args['py_modules'] = modulesToInstall
+    args['data_files'] = getDataFiles('twisted') + _dataFiles
+    args['zip_safe'] = False
     args['cmdclass'] = {'sdist': DisabledSdist}
+    args['scripts'] = ['bin/trial', 'bin/twistd']
 
     setup(**args)
 
