@@ -26,11 +26,6 @@ except ImportError:
 else:
     shadow = None
 
-try:
-    from twisted.cred import pamauth
-except ImportError:
-    pamauth = None
-
 from zope.interface import providedBy, implementer, Interface
 
 
@@ -206,7 +201,7 @@ class SSHPublicKeyDatabase:
                 continue
             try:
                 lines = filepath.open()
-            except IOError, e:
+            except IOError as e:
                 if e.errno == errno.EACCES:
                     lines = runAsEffectiveUser(ouid, ogid, filepath.open)
                 else:
