@@ -12,7 +12,7 @@ import hmac
 from binascii import Error as DecodeError, b2a_base64
 from hashlib import sha1
 
-from zope.interface import implements
+from zope.interface import implementer
 
 from twisted.python.randbytes import secureRandom
 from twisted.internet import defer
@@ -102,6 +102,7 @@ class _BaseEntry(object):
 
 
 
+@implementer(IKnownHostEntry)
 class PlainEntry(_BaseEntry):
     """
     A L{PlainEntry} is a representation of a plain-text entry in a known_hosts
@@ -111,8 +112,6 @@ class PlainEntry(_BaseEntry):
     @type _hostnames: L{list} of L{str}
     """
 
-    implements(IKnownHostEntry)
-
     def __init__(self, hostnames, keyType, publicKey, comment):
         self._hostnames = hostnames
         super(PlainEntry, self).__init__(keyType, publicKey, comment)
@@ -181,14 +180,13 @@ class PlainEntry(_BaseEntry):
 
 
 
+@implementer(IKnownHostEntry)
 class UnparsedEntry(object):
     """
     L{UnparsedEntry} is an entry in a L{KnownHostsFile} which can't actually be
     parsed; therefore it matches no keys and no hosts.
     """
 
-    implements(IKnownHostEntry)
-
     def __init__(self, string):
         """
         Create an unparsed entry from a line in a known_hosts file which cannot
@@ -242,6 +240,7 @@ def _hmacedString(key, string):
 
 
 
+@implementer(IKnownHostEntry)
 class HashedEntry(_BaseEntry, FancyEqMixin):
     """
     A L{HashedEntry} is a representation of an entry in a known_hosts file
@@ -255,8 +254,6 @@ class HashedEntry(_BaseEntry, FancyEqMixin):
     known_hosts file as opposed to a plaintext one.
     """
 
-    implements(IKnownHostEntry)
-
     MAGIC = '|1|'
 
     compareAttributes = (
