@@ -13,9 +13,14 @@ else:
     cryptSkip = None
 
 import os, base64
+from collections import namedtuple
+from io import StringIO
+
+from zope.interface.verify import verifyObject
 
 from twisted.python import util
 from twisted.python.failure import Failure
+from twisted.python.reflect import requireModule
 from twisted.trial.unittest import TestCase
 from twisted.python.filepath import FilePath
 from twisted.cred.checkers import InMemoryUsernamePasswordDatabaseDontUse
@@ -25,17 +30,14 @@ from twisted.cred.error import UnhandledCredentials, UnauthorizedLogin
 from twisted.python.fakepwd import UserDatabase, ShadowDatabase
 from twisted.test.test_process import MockOS
 
-try:
-    import Crypto.Cipher.DES3
-    import pyasn1
-except ImportError:
-    dependencySkip = "can't run without Crypto and PyASN1"
-else:
+if requireModule('cryptography') and requireModule('pyasn1'):
     dependencySkip = None
     from twisted.conch.ssh import keys
     from twisted.conch import checkers
     from twisted.conch.error import NotEnoughAuthentication, ValidPublicKey
     from twisted.conch.test import keydata
+else:
+    dependencySkip = "can't run without cryptography and PyASN1"
 
 if getattr(os, 'geteuid', None) is None:
     euidSkip = "Cannot run without effective UIDs (questionable)"
@@ -171,7 +173,7 @@ class HelperTests(TestCase):
 
 
 
-class SSHPublicKeyDatabaseTestCase(TestCase):
+class SSHPublicKeyDatabaseTests(TestCase):
     """
     Tests for L{SSHPublicKeyDatabase}.
     """
@@ -197,6 +199,23 @@ class SSHPublicKeyDatabaseTestCase(TestCase):
         self.checker._userdb = userdb
 
 
+    def test_deprecated(self):
+        """
+        L{SSHPublicKeyDatabase} is deprecated as of version 15.0
+        """
+        warningsShown = self.flushWarnings(
+            offendingFunctions=[self.setUp])
+        self.assertEqual(warningsShown[0]['category'], DeprecationWarning)
+        self.assertEqual(
+            warningsShown[0]['message'],
+            "twisted.conch.checkers.SSHPublicKeyDatabase "
+            "was deprecated in Twisted 15.0.0: Please use "
+            "twisted.conch.checkers.SSHPublicKeyChecker, "
+            "initialized with an instance of "
+            "twisted.conch.checkers.UNIXAuthorizedKeysFiles instead.")
+        self.assertEqual(len(warningsShown), 1)
+
+
     def _testCheckKey(self, filename):
         self.sshDir.child(filename).setContent(self.content)
         user = UsernamePassword("user", "password")
@@ -334,7 +353,7 @@ class SSHPublicKeyDatabaseTestCase(TestCase):
 
 
 
-class SSHProtocolCheckerTestCase(TestCase):
+class SSHProtocolCheckerTests(TestCase):
     """
     Tests for L{SSHProtocolChecker}.
     """
@@ -601,3 +620,273 @@ class UNIXPasswordDatabaseTests(TestCase):
 
         cred = UsernamePassword('carol', '*')
         self.assertUnauthorizedLogin(checker.requestAvatarId(cred))
+
+
+
+class AuthorizedKeyFileReaderTests(TestCase):
+    """
+    Tests for L{checkers.readAuthorizedKeyFile}
+    """
+    skip = dependencySkip
+
+
+    def test_ignoresComments(self):
+        """
+        L{checkers.readAuthorizedKeyFile} does not attempt to turn comments
+        into keys
+        """
+        fileobj = StringIO(u'# this comment is ignored\n'
+                           u'this is not\n'
+                           u'# this is again\n'
+                           u'and this is not')
+        result = checkers.readAuthorizedKeyFile(fileobj, lambda x: x)
+        self.assertEqual(['this is not', 'and this is not'], list(result))
+
+
+    def test_ignoresLeadingWhitespaceAndEmptyLines(self):
+        """
+        L{checkers.readAuthorizedKeyFile} ignores leading whitespace in
+        lines, as well as empty lines
+        """
+        fileobj = StringIO(u"""
+                           # ignore
+                           not ignored
+                           """)
+        result = checkers.readAuthorizedKeyFile(fileobj, parseKey=lambda x: x)
+        self.assertEqual(['not ignored'], list(result))
+
+
+    def test_ignoresUnparsableKeys(self):
+        """
+        L{checkers.readAuthorizedKeyFile} does not raise an exception
+        when a key fails to parse (raises a
+        L{twisted.conch.ssh.keys.BadKeyError}), but rather just keeps going
+        """
+        def failOnSome(line):
+            if line.startswith('f'):
+                raise keys.BadKeyError('failed to parse')
+            return line
+
+        fileobj = StringIO(u'failed key\ngood key')
+        result = checkers.readAuthorizedKeyFile(fileobj,
+                                                parseKey=failOnSome)
+        self.assertEqual(['good key'], list(result))
+
+
+
+class InMemorySSHKeyDBTests(TestCase):
+    """
+    Tests for L{checkers.InMemorySSHKeyDB}
+    """
+    skip = dependencySkip
+
+
+    def test_implementsInterface(self):
+        """
+        L{checkers.InMemorySSHKeyDB} implements
+        L{checkers.IAuthorizedKeysDB}
+        """
+        keydb = checkers.InMemorySSHKeyDB({'alice': ['key']})
+        verifyObject(checkers.IAuthorizedKeysDB, keydb)
+
+
+    def test_noKeysForUnauthorizedUser(self):
+        """
+        If the user is not in the mapping provided to
+        L{checkers.InMemorySSHKeyDB}, an empty iterator is returned
+        by L{checkers.InMemorySSHKeyDB.getAuthorizedKeys}
+        """
+        keydb = checkers.InMemorySSHKeyDB({'alice': ['keys']})
+        self.assertEqual([], list(keydb.getAuthorizedKeys('bob')))
+
+
+    def test_allKeysForAuthorizedUser(self):
+        """
+        If the user is in the mapping provided to
+        L{checkers.InMemorySSHKeyDB}, an iterator with all the keys
+        is returned by L{checkers.InMemorySSHKeyDB.getAuthorizedKeys}
+        """
+        keydb = checkers.InMemorySSHKeyDB({'alice': ['a', 'b']})
+        self.assertEqual(['a', 'b'], list(keydb.getAuthorizedKeys('alice')))
+
+
+
+class UNIXAuthorizedKeysFilesTests(TestCase):
+    """
+    Tests for L{checkers.UNIXAuthorizedKeysFiles}.
+    """
+    skip = dependencySkip
+
+
+    def setUp(self):
+        mockos = MockOS()
+        mockos.path = FilePath(self.mktemp())
+        mockos.path.makedirs()
+
+        self.userdb = UserDatabase()
+        self.userdb.addUser('alice', 'password', 1, 2, 'alice lastname',
+                            mockos.path.path, '/bin/shell')
+
+        self.sshDir = mockos.path.child('.ssh')
+        self.sshDir.makedirs()
+        authorizedKeys = self.sshDir.child('authorized_keys')
+        authorizedKeys.setContent('key 1\nkey 2')
+
+        self.expectedKeys = ['key 1', 'key 2']
+
+
+    def test_implementsInterface(self):
+        """
+        L{checkers.UNIXAuthorizedKeysFiles} implements
+        L{checkers.IAuthorizedKeysDB}.
+        """
+        keydb = checkers.UNIXAuthorizedKeysFiles(self.userdb)
+        verifyObject(checkers.IAuthorizedKeysDB, keydb)
+
+
+    def test_noKeysForUnauthorizedUser(self):
+        """
+        If the user is not in the user database provided to
+        L{checkers.UNIXAuthorizedKeysFiles}, an empty iterator is returned
+        by L{checkers.UNIXAuthorizedKeysFiles.getAuthorizedKeys}.
+        """
+        keydb = checkers.UNIXAuthorizedKeysFiles(self.userdb,
+                                                 parseKey=lambda x: x)
+        self.assertEqual([], list(keydb.getAuthorizedKeys('bob')))
+
+
+    def test_allKeysInAllAuthorizedFilesForAuthorizedUser(self):
+        """
+        If the user is in the user database provided to
+        L{checkers.UNIXAuthorizedKeysFiles}, an iterator with all the keys in
+        C{~/.ssh/authorized_keys} and C{~/.ssh/authorized_keys2} is returned
+        by L{checkers.UNIXAuthorizedKeysFiles.getAuthorizedKeys}.
+        """
+        self.sshDir.child('authorized_keys2').setContent('key 3')
+        keydb = checkers.UNIXAuthorizedKeysFiles(self.userdb,
+                                                 parseKey=lambda x: x)
+        self.assertEqual(self.expectedKeys + ['key 3'],
+                         list(keydb.getAuthorizedKeys('alice')))
+
+
+    def test_ignoresNonexistantFile(self):
+        """
+        L{checkers.UNIXAuthorizedKeysFiles.getAuthorizedKeys} returns only
+        the keys in C{~/.ssh/authorized_keys} and C{~/.ssh/authorized_keys2}
+        if they exist.
+        """
+        keydb = checkers.UNIXAuthorizedKeysFiles(self.userdb,
+                                                 parseKey=lambda x: x)
+        self.assertEqual(self.expectedKeys,
+                         list(keydb.getAuthorizedKeys('alice')))
+
+
+    def test_ignoresUnreadableFile(self):
+        """
+        L{checkers.UNIXAuthorizedKeysFiles.getAuthorizedKeys} returns only
+        the keys in C{~/.ssh/authorized_keys} and C{~/.ssh/authorized_keys2}
+        if they are readable.
+        """
+        self.sshDir.child('authorized_keys2').makedirs()
+        keydb = checkers.UNIXAuthorizedKeysFiles(self.userdb,
+                                                 parseKey=lambda x: x)
+        self.assertEqual(self.expectedKeys,
+                         list(keydb.getAuthorizedKeys('alice')))
+
+
+
+_KeyDB = namedtuple('KeyDB', ['getAuthorizedKeys'])
+
+
+
+class _DummyException(Exception):
+    """
+    Fake exception to be used for testing.
+    """
+    pass
+
+
+
+class SSHPublicKeyCheckerTests(TestCase):
+    """
+    Tests for L{checkers.SSHPublicKeyChecker}.
+    """
+    skip = dependencySkip
+
+
+    def setUp(self):
+        self.credentials = SSHPrivateKey(
+            'alice', 'ssh-rsa', keydata.publicRSA_openssh, 'foo',
+             keys.Key.fromString(keydata.privateRSA_openssh).sign('foo'))
+        self.keydb = _KeyDB(lambda _: [
+            keys.Key.fromString(keydata.publicRSA_openssh)])
+        self.checker = checkers.SSHPublicKeyChecker(self.keydb)
+
+
+    def test_credentialsWithoutSignature(self):
+        """
+        Calling L{checkers.SSHPublicKeyChecker.requestAvatarId} with
+        credentials that do not have a signature fails with L{ValidPublicKey}.
+        """
+        self.credentials.signature = None
+        self.failureResultOf(self.checker.requestAvatarId(self.credentials),
+                             ValidPublicKey)
+
+
+    def test_credentialsWithBadKey(self):
+        """
+        Calling L{checkers.SSHPublicKeyChecker.requestAvatarId} with
+        credentials that have a bad key fails with L{keys.BadKeyError}.
+        """
+        self.credentials.blob = ''
+        self.failureResultOf(self.checker.requestAvatarId(self.credentials),
+                             keys.BadKeyError)
+
+
+    def test_credentialsNoMatchingKey(self):
+        """
+        If L{checkers.IAuthorizedKeysDB.getAuthorizedKeys} returns no keys
+        that match the credentials,
+        L{checkers.SSHPublicKeyChecker.requestAvatarId} fails with
+        L{UnauthorizedLogin}.
+        """
+        self.credentials.blob = keydata.publicDSA_openssh
+        self.failureResultOf(self.checker.requestAvatarId(self.credentials),
+                             UnauthorizedLogin)
+
+
+    def test_credentialsInvalidSignature(self):
+        """
+        Calling L{checkers.SSHPublicKeyChecker.requestAvatarId} with
+        credentials that are incorrectly signed fails with
+        L{UnauthorizedLogin}.
+        """
+        self.credentials.signature = (
+            keys.Key.fromString(keydata.privateDSA_openssh).sign('foo'))
+        self.failureResultOf(self.checker.requestAvatarId(self.credentials),
+                             UnauthorizedLogin)
+
+
+    def test_failureVerifyingKey(self):
+        """
+        If L{keys.Key.verify} raises an exception,
+        L{checkers.SSHPublicKeyChecker.requestAvatarId} fails with
+        L{UnauthorizedLogin}.
+        """
+        def fail(*args, **kwargs):
+            raise _DummyException()
+
+        self.patch(keys.Key, 'verify', fail)
+
+        self.failureResultOf(self.checker.requestAvatarId(self.credentials),
+                             UnauthorizedLogin)
+        self.flushLoggedErrors(_DummyException)
+
+
+    def test_usernameReturnedOnSuccess(self):
+        """
+        L{checker.SSHPublicKeyChecker.requestAvatarId}, if successful,
+        callbacks with the username.
+        """
+        d = self.checker.requestAvatarId(self.credentials)
+        self.assertEqual('alice', self.successResultOf(d))
