@@ -183,10 +183,10 @@ class TestConnection(connection.SSHConnection):
 
 
 
-class ConnectionTestCase(unittest.TestCase):
+class ConnectionTests(unittest.TestCase):
 
     if test_userauth.transport is None:
-        skip = "Cannot run without both PyCrypto and pyasn1"
+        skip = "Cannot run without both cryptography and pyasn1"
 
     def setUp(self):
         self.transport = test_userauth.FakeTransport(None)
@@ -700,12 +700,12 @@ class ConnectionTestCase(unittest.TestCase):
 
 
 
-class TestCleanConnectionShutdown(unittest.TestCase):
+class CleanConnectionShutdownTests(unittest.TestCase):
     """
     Check whether correct cleanup is performed on connection shutdown.
     """
     if test_userauth.transport is None:
-        skip = "Cannot run without both PyCrypto and pyasn1"
+        skip = "Cannot run without both cryptography and pyasn1"
 
     def setUp(self):
         self.transport = test_userauth.FakeTransport(None)
