@@ -137,6 +137,20 @@ def simpleVerifyHostname(connection, hostname):
 
 
 
+def _usablePyOpenSSL(version):
+    """
+    Check pyOpenSSL version string whether we can use it for host verification.
+
+    @param version: A pyOpenSSL version string.
+    @type version: L{str}
+
+    @rtype: L{bool}
+    """
+    major, minor = (int(part) for part in version.split(".")[:2])
+    return (major, minor) >= (0, 12)
+
+
+
 def _selectVerifyImplementation(lib):
     """
     U{service_identity <https://pypi.python.org/pypi/service_identity>}
@@ -160,9 +174,7 @@ def _selectVerifyImplementation(lib):
         "rejected."
     )
 
-    major, minor = list(int(part) for part in lib.__version__.split("."))[:2]
-
-    if (major, minor) >= (0, 12):
+    if _usablePyOpenSSL(lib.__version__):
         try:
             from service_identity import VerificationError
             from service_identity.pyopenssl import verify_hostname
@@ -192,7 +204,6 @@ def _selectVerifyImplementation(lib):
 verifyHostname, VerificationError = _selectVerifyImplementation(OpenSSL)
 
 
-
 from zope.interface import Interface, implementer
 
 from twisted.internet.defer import Deferred
@@ -204,6 +215,7 @@ from twisted.internet.interfaces import (
 from twisted.python import reflect, util
 from twisted.python.deprecate import _mutuallyExclusiveArguments
 from twisted.python.compat import nativeString, networkString, unicode
+from twisted.python.constants import Flags, FlagConstant
 from twisted.python.failure import Failure
 from twisted.python.util import FancyEqMixin
 
@@ -220,6 +232,60 @@ def _sessionCounter(counter=itertools.count()):
 
 
 
+class ProtocolNegotiationSupport(Flags):
+    """
+    L{ProtocolNegotiationSupport} defines flags which are used to indicate the
+    level of NPN/ALPN support provided by the TLS backend.
+
+    @cvar NOSUPPORT: There is no support for NPN or ALPN. This is exclusive
+        with both L{NPN} and L{ALPN}.
+    @cvar NPN: The implementation supports Next Protocol Negotiation.
+    @cvar ALPN: The implementation supports Application Layer Protocol
+        Negotiation.
+    """
+    NPN = FlagConstant(0x0001)
+    ALPN = FlagConstant(0x0002)
+
+# FIXME: https://twistedmatrix.com/trac/ticket/8074
+# Currently flags with literal zero values behave incorrectly. However,
+# creating a flag by NOTing a flag with itself appears to work totally fine, so
+# do that instead.
+ProtocolNegotiationSupport.NOSUPPORT = (
+    ProtocolNegotiationSupport.NPN ^ ProtocolNegotiationSupport.NPN
+)
+
+
+def protocolNegotiationMechanisms():
+    """
+    Checks whether your versions of PyOpenSSL and OpenSSL are recent enough to
+    support protocol negotiation, and if they are, what kind of protocol
+    negotiation is supported.
+
+    @return: A combination of flags from L{ProtocolNegotiationSupport} that
+        indicate which mechanisms for protocol negotiation are supported.
+    @rtype: L{FlagConstant}
+    """
+    support = ProtocolNegotiationSupport.NOSUPPORT
+    ctx = SSL.Context(SSL.SSLv23_METHOD)
+
+    try:
+        ctx.set_npn_advertise_callback(lambda c: None)
+    except (AttributeError, NotImplementedError):
+        pass
+    else:
+        support |= ProtocolNegotiationSupport.NPN
+
+    try:
+        ctx.set_alpn_select_callback(lambda c: None)
+    except (AttributeError, NotImplementedError):
+        pass
+    else:
+        support |= ProtocolNegotiationSupport.ALPN
+
+    return support
+
+
+
 _x509names = {
     'CN': 'commonName',
     'commonName': 'commonName',
@@ -657,7 +723,7 @@ class PrivateCertificate(Certificate):
 
 
     def certificateRequest(self, format=crypto.FILETYPE_ASN1,
-                           digestAlgorithm='md5'):
+                           digestAlgorithm='sha256'):
         return self.privateKey.certificateRequest(
             self.getSubject(),
             format,
@@ -682,7 +748,7 @@ class PrivateCertificate(Certificate):
 
     def signRequestObject(self, certificateRequest, serialNumber,
                           secondsToExpiry=60 * 60 * 24 * 365, # One year
-                          digestAlgorithm='md5'):
+                          digestAlgorithm='sha256'):
         return self.privateKey.signRequestObject(self.getSubject(),
                                                  certificateRequest,
                                                  serialNumber,
@@ -816,7 +882,7 @@ class KeyPair(PublicKey):
     generate = classmethod(generate)
 
 
-    def requestObject(self, distinguishedName, digestAlgorithm='md5'):
+    def requestObject(self, distinguishedName, digestAlgorithm='sha256'):
         req = crypto.X509Req()
         req.set_pubkey(self.original)
         distinguishedName._copyInto(req.get_subject())
@@ -826,7 +892,7 @@ class KeyPair(PublicKey):
 
     def certificateRequest(self, distinguishedName,
                            format=crypto.FILETYPE_ASN1,
-                           digestAlgorithm='md5'):
+                           digestAlgorithm='sha256'):
         """Create a certificate request signed with this key.
 
         @return: a string, formatted according to the 'format' argument.
@@ -842,7 +908,7 @@ class KeyPair(PublicKey):
                                requestFormat=crypto.FILETYPE_ASN1,
                                certificateFormat=crypto.FILETYPE_ASN1,
                                secondsToExpiry=60 * 60 * 24 * 365, # One year
-                               digestAlgorithm='md5'):
+                               digestAlgorithm='sha256'):
         """
         Given a blob of certificate request data and a certificate authority's
         DistinguishedName, return a blob of signed certificate data.
@@ -872,7 +938,7 @@ class KeyPair(PublicKey):
                           requestObject,
                           serialNumber,
                           secondsToExpiry=60 * 60 * 24 * 365, # One year
-                          digestAlgorithm='md5'):
+                          digestAlgorithm='sha256'):
         """
         Sign a CertificateRequest instance, returning a Certificate instance.
         """
@@ -946,6 +1012,39 @@ class OpenSSLCertificateAuthorities(object):
 
 
 
+def trustRootFromCertificates(certificates):
+    """
+    Builds an object that trusts multiple root L{Certificate}s.
+
+    When passed to L{optionsForClientTLS}, connections using those options will
+    reject any server certificate not signed by at least one of the
+    certificates in the `certificates` list.
+
+    @since: 16.0.0
+
+    @param certificates: All certificates which will be trusted.
+    @type certificates: C{iterable} of L{CertBase}
+
+    @rtype: L{IOpenSSLTrustRoot}
+    @return: an object suitable for use as the trustRoot= keyword argument to
+        L{optionsForClientTLS}
+    """
+
+    certs = []
+    for cert in certificates:
+        # PrivateCertificate or Certificate are both okay
+        if isinstance(cert, CertBase):
+            cert = cert.original
+        else:
+            raise TypeError(
+                "certificates items must be twisted.internet.ssl.CertBase"
+                " instances"
+            )
+        certs.append(cert)
+    return OpenSSLCertificateAuthorities(certs)
+
+
+
 @implementer(IOpenSSLTrustRoot)
 class OpenSSLDefaultPaths(object):
     """
@@ -1160,7 +1259,7 @@ class ClientTLSOptions(object):
 
 
 def optionsForClientTLS(hostname, trustRoot=None, clientCertificate=None,
-                         **kw):
+                        acceptableProtocols=None, **kw):
     """
     Create a L{client connection creator <IOpenSSLClientConnectionCreator>} for
     use with APIs such as L{SSL4ClientEndpoint
@@ -1192,6 +1291,15 @@ def optionsForClientTLS(hostname, trustRoot=None, clientCertificate=None,
         will not authenticate.
     @type clientCertificate: L{PrivateCertificate}
 
+    @param acceptableProtocols: The protocols this peer is willing to speak
+        after the TLS negotation has completed, advertised over both ALPN and
+        NPN. If this argument is specified, and no overlap can be found with
+        the other peer, the connection will fail to be established. If the
+        remote peer does not offer NPN or ALPN, the connection will be
+        established, but no protocol wil be negotiated. Protocols earlier in
+        the list are preferred over those later in the list.
+    @type acceptableProtocols: C{list} of C{bytes}
+
     @param extraCertificateOptions: keyword-only argument; this is a dictionary
         of additional keyword arguments to be presented to
         L{CertificateOptions}.  Please avoid using this unless you absolutely
@@ -1228,6 +1336,7 @@ def optionsForClientTLS(hostname, trustRoot=None, clientCertificate=None,
         )
     certificateOptions = OpenSSLCertificateOptions(
         trustRoot=trustRoot,
+        acceptableProtocols=acceptableProtocols,
         **extraCertificateOptions
     )
     return ClientTLSOptions(hostname, certificateOptions.getContext())
@@ -1281,7 +1390,9 @@ class OpenSSLCertificateOptions(object):
                  extraCertChain=None,
                  acceptableCiphers=None,
                  dhParameters=None,
-                 trustRoot=None):
+                 trustRoot=None,
+                 acceptableProtocols=None,
+                 ):
         """
         Create an OpenSSL context SSL connection context factory.
 
@@ -1372,6 +1483,15 @@ class OpenSSLCertificateOptions(object):
 
         @type trustRoot: L{IOpenSSLTrustRoot}
 
+        @param acceptableProtocols: The protocols this peer is willing to speak
+            after the TLS negotation has completed, advertised over both ALPN
+            and NPN. If this argument is specified, and no overlap can be found
+            with the other peer, the connection will fail to be established.
+            If the remote peer does not offer NPN or ALPN, the connection will
+            be established, but no protocol wil be negotiated. Protocols
+            earlier in the list are preferred over those later in the list.
+        @type acceptableProtocols: C{list} of C{bytes}
+
         @raise ValueError: when C{privateKey} or C{certificate} are set without
             setting the respective other.
         @raise ValueError: when C{verify} is L{True} but C{caCerts} doesn't
@@ -1384,6 +1504,8 @@ class OpenSSLCertificateOptions(object):
         @raise TypeError: if C{trustRoot} is passed in combination with
             C{caCert}, C{verify}, or C{requireCertificate}.  Please prefer
             C{trustRoot} in new code, as its semantics are less tricky.
+        @raises NotImplementedError: If acceptableProtocols were provided but
+            no negotiation mechanism is available.
         """
 
         if (privateKey is None) != (certificate is None):
@@ -1467,6 +1589,13 @@ class OpenSSLCertificateOptions(object):
             trustRoot = IOpenSSLTrustRoot(trustRoot)
         self.trustRoot = trustRoot
 
+        if acceptableProtocols is not None and not protocolNegotiationMechanisms():
+            raise NotImplementedError(
+                "No support for protocol negotiation on this platform."
+            )
+
+        self._acceptableProtocols = acceptableProtocols
+
 
     def __getstate__(self):
         d = self.__dict__.copy()
@@ -1536,9 +1665,54 @@ class OpenSSLCertificateOptions(object):
             except BaseException:
                 pass  # ECDHE support is best effort only.
 
+        if self._acceptableProtocols:
+            # Try to set NPN and ALPN. _acceptableProtocols cannot be set by
+            # the constructor unless at least one mechanism is supported.
+            self._setUpNextProtocolMechanisms(ctx)
+
         return ctx
 
 
+    def _setUpNextProtocolMechanisms(self, ctx):
+        """
+        Called to set up the C{ctx} for doing NPN and/or ALPN negotiation.
+
+        @param ctx: The context which is set up.
+        @type ctx: L{OpenSSL.SSL.Context}
+        """
+        supported = protocolNegotiationMechanisms()
+
+        if supported & ProtocolNegotiationSupport.NPN:
+            def npnAdvertiseCallback(conn):
+                return self._acceptableProtocols
+
+            ctx.set_npn_advertise_callback(npnAdvertiseCallback)
+            ctx.set_npn_select_callback(self._protoSelectCallback)
+
+        if supported & ProtocolNegotiationSupport.ALPN:
+            ctx.set_alpn_select_callback(self._protoSelectCallback)
+            ctx.set_alpn_protos(self._acceptableProtocols)
+
+
+    def _protoSelectCallback(self, conn, protocols):
+        """
+        NPN client-side and ALPN server-side callback used to select
+        the next protocol. Prefers protocols found earlier in
+        C{_acceptableProtocols}.
+
+        @param conn: The context which is set up.
+        @type conn: L{OpenSSL.SSL.Connection}
+
+        @param conn: Protocols advertised by the other side.
+        @type conn: C{list} of C{bytes}
+        """
+        overlap = set(protocols) & set(self._acceptableProtocols)
+
+        for p in self._acceptableProtocols:
+            if p in overlap:
+                return p
+        else:
+            return b''
 
 OpenSSLCertificateOptions.__getstate__ = deprecated(
         Version("Twisted", 15, 0, 0),
