@@ -18,6 +18,7 @@ from functools import wraps
 from zope.interface import implementer
 from zope.interface.verify import verifyClass
 
+from twisted.python.compat import long
 from twisted.python.runtime import platform
 from twisted.python.failure import Failure
 from twisted.python import log
@@ -2513,8 +2514,8 @@ class SimpleUtilityTests(TestCase):
         # integers, because the whole point of getaddrinfo is that you can never
         # know a-priori know _anything_ about the network interfaces of the
         # computer that you're on and you have to ask it.
-        self.assertIsInstance(result[2], int) # flow info
-        self.assertIsInstance(result[3], int) # scope id
+        self.assertIsInstance(result[2], (int, long)) # flow info
+        self.assertIsInstance(result[3], (int, long)) # scope id
         # but, luckily, IP presentation format and what it means to be a port
         # number are a little better specified.
         self.assertEqual(result[:2], ("::1", 2))
