@@ -18,9 +18,10 @@ See L{twisted.cred.credentials} and L{twisted.cred._digest} for its new home.
 import socket, time, sys, random, warnings
 from hashlib import md5
 from zope.interface import implements, Interface
+from collections import OrderedDict
 
 # twisted imports
-from twisted.python import log, util
+from twisted.python import log
 from twisted.python.deprecate import deprecated
 from twisted.python.versions import Version
 from twisted.internet import protocol, defer, reactor
@@ -539,7 +540,7 @@ class Message:
     length = None
 
     def __init__(self):
-        self.headers = util.OrderedDict() # map name to list of values
+        self.headers = OrderedDict() # map name to list of values
         self.body = ""
         self.finished = 0
 
