@@ -9,31 +9,32 @@ to test that IHalfCloseableProtocol.readConnectionLost works for process
 transports.
 """
 
+from __future__ import absolute_import, division
+
 __import__('_preamble')
 import sys
 
-from zope.interface import implements
+from zope.interface import implementer
 
 from twisted.internet.interfaces import IHalfCloseableProtocol
 from twisted.internet import stdio, protocol
 from twisted.python import reflect, log
 
 
+@implementer(IHalfCloseableProtocol)
 class HalfCloseProtocol(protocol.Protocol):
     """
     A protocol to hook up to stdio and observe its transport being
     half-closed.  If all goes as expected, C{exitCode} will be set to C{0};
     otherwise it will be set to C{1} to indicate failure.
     """
-    implements(IHalfCloseableProtocol)
-
     exitCode = None
 
     def connectionMade(self):
         """
         Signal the parent process that we're ready.
         """
-        self.transport.write("x")
+        self.transport.write(b"x")
 
 
     def readConnectionLost(self):
@@ -59,7 +60,7 @@ class HalfCloseProtocol(protocol.Protocol):
 
 if __name__ == '__main__':
     reflect.namedAny(sys.argv[1]).install()
-    log.startLogging(file(sys.argv[2], 'w'))
+    log.startLogging(open(sys.argv[2], 'wb'))
     from twisted.internet import reactor
     protocol = HalfCloseProtocol()
     stdio.StandardIO(protocol)
