@@ -8,16 +8,21 @@ L{twisted.test.test_stdio.StandardInputOutputTests.test_write} to test that
 ITransport.write() works for process transports.
 """
 
+from __future__ import absolute_import, division
+
 __import__('_preamble')
 import sys
 
 from twisted.internet import stdio, protocol
 from twisted.python import reflect
 
+
+
 class WriteChild(protocol.Protocol):
     def connectionMade(self):
-        for ch in 'ok!':
-            self.transport.write(ch)
+        self.transport.write(b'o')
+        self.transport.write(b'k')
+        self.transport.write(b'!')
         self.transport.loseConnection()
 
 
@@ -25,6 +30,7 @@ class WriteChild(protocol.Protocol):
         reactor.stop()
 
 
+
 if __name__ == '__main__':
     reflect.namedAny(sys.argv[1]).install()
     from twisted.internet import reactor
