@@ -5,12 +5,13 @@
 Tests for L{twisted.application.app} and L{twisted.scripts.twistd}.
 """
 
+from __future__ import absolute_import, division
+
 import errno
 import inspect
 import signal
 import os
 import sys
-import StringIO
 
 try:
     import pwd
@@ -23,7 +24,7 @@ try:
 except ImportError:
     import pickle
 
-from zope.interface import implements
+from zope.interface import implementer
 from zope.interface.verify import verifyObject
 
 from twisted.trial import unittest
@@ -34,6 +35,7 @@ from twisted.application.service import IServiceMaker
 from twisted.application import service, app, reactors
 from twisted.scripts import twistd
 from twisted.python import log
+from twisted.python.compat import NativeStringIO
 from twisted.python.usage import UsageError
 from twisted.python.log import ILogObserver
 from twisted.python.components import Componentized
@@ -61,14 +63,6 @@ try:
 except ImportError:
     profile = None
 
-try:
-    import hotshot
-    import hotshot.stats
-except (ImportError, SystemExit):
-    # For some reasons, hotshot.stats seems to raise SystemExit on some
-    # distributions, probably when considered non-free.  See the import of
-    # this module in twisted.application.app for more details.
-    hotshot = None
 
 try:
     import pstats
@@ -242,7 +236,7 @@ class ServerOptionsTests(unittest.TestCase):
         config = twistd.ServerOptions()
         self.assertEqual(config._getReactorTypes, reactors.getReactorTypes)
         config._getReactorTypes = getReactorTypes
-        config.messageOutput = StringIO.StringIO()
+        config.messageOutput = NativeStringIO()
 
         self.assertRaises(SystemExit, config.parseOptions, ['--help-reactors'])
         helpOutput = config.messageOutput.getvalue()
@@ -347,10 +341,17 @@ class ServerOptionsTests(unittest.TestCase):
         config = twistd.ServerOptions()
         e = self.assertRaises(UsageError, config.parseOptions,
                               ["--logger", "twisted.test.test_twistd.FOOBAR"])
-        self.assertTrue(
-            e.args[0].startswith(
-                "Logger 'twisted.test.test_twistd.FOOBAR' could not be "
-                "imported: 'module' object has no attribute 'FOOBAR'"))
+        if sys.version_info <= (3, 5):
+            self.assertTrue(
+                e.args[0].startswith(
+                    "Logger 'twisted.test.test_twistd.FOOBAR' could not be "
+                    "imported: 'module' object has no attribute 'FOOBAR'"))
+        else:
+            self.assertTrue(
+                e.args[0].startswith(
+                    "Logger 'twisted.test.test_twistd.FOOBAR' could not be "
+                    "imported: module 'twisted.test.test_twistd' "
+                    "has no attribute 'FOOBAR'"))
         self.assertNotIn('\n', e.args[0])
 
 
@@ -365,9 +366,8 @@ class TapFileTests(unittest.TestCase):
         Create a trivial Application and put it in a tap file on disk.
         """
         self.tapfile = self.mktemp()
-        f = file(self.tapfile, 'wb')
-        pickle.dump(service.Application("Hi!"), f)
-        f.close()
+        with open(self.tapfile, 'wb') as f:
+            pickle.dump(service.Application("Hi!"), f)
 
 
     def test_createOrGetApplicationWithTapFile(self):
@@ -505,9 +505,8 @@ class ApplicationRunnerTests(unittest.TestCase):
             def removePID(self, pidfile):
                 pass
 
-
+        @implementer(service.IService, service.IProcess)
         class FakeService(object):
-            implements(service.IService, service.IProcess)
 
             processName = None
             uid = None
@@ -692,9 +691,8 @@ class UnixApplicationRunnerSetupEnvironmentTests(unittest.TestCase):
         """
         pidfile = self.mktemp()
         self.runner.setupEnvironment(None, ".", True, None, pidfile)
-        fObj = file(pidfile)
-        pid = int(fObj.read())
-        fObj.close()
+        with open(pidfile, 'rb') as f:
+            pid = int(f.read())
         self.assertEqual(pid, self.pid)
 
 
@@ -707,9 +705,8 @@ class UnixApplicationRunnerSetupEnvironmentTests(unittest.TestCase):
         pidfile = self.mktemp()
         with AlternateReactor(FakeDaemonizingReactor()):
             self.runner.setupEnvironment(None, ".", False, None, pidfile)
-        fObj = file(pidfile)
-        pid = int(fObj.read())
-        fObj.close()
+        with open(pidfile, 'rb') as f:
+            pid = int(f.read())
         self.assertEqual(pid, self.pid + 1)
 
 
@@ -741,7 +738,7 @@ class UnixApplicationRunnerSetupEnvironmentTests(unittest.TestCase):
         """
         with AlternateReactor(FakeDaemonizingReactor()):
             self.runner.setupEnvironment(None, ".", False, None, None)
-        self.assertEqual(self.mask, 0077)
+        self.assertEqual(self.mask, 0o077)
 
 
 
@@ -810,7 +807,7 @@ class UnixApplicationRunnerRemovePIDTests(unittest.TestCase):
         path = self.mktemp()
         os.makedirs(path)
         pidfile = os.path.join(path, "foo.pid")
-        file(pidfile, "w").close()
+        open(pidfile, "w").close()
         runner.removePID(pidfile)
         self.assertFalse(os.path.exists(pidfile))
 
@@ -860,12 +857,12 @@ class FakeNonDaemonizingReactor(object):
 
 
 
+@implementer(IReactorDaemonize)
 class FakeDaemonizingReactor(FakeNonDaemonizingReactor):
     """
     A dummy reactor, providing C{beforeDaemonize} and C{afterDaemonize}
     methods, announcing this, and logging whether the methods have been called.
     """
-    implements(IReactorDaemonize)
 
 
 
@@ -908,7 +905,8 @@ class AppProfilingTests(unittest.TestCase):
         profiler.run(reactor)
 
         self.assertTrue(reactor.called)
-        data = file(config["profile"]).read()
+        with open(config["profile"]) as f:
+            data = f.read()
         self.assertIn("DummyReactor.run", data)
         self.assertIn("function calls", data)
 
@@ -917,7 +915,7 @@ class AppProfilingTests(unittest.TestCase):
 
 
     def _testStats(self, statsClass, profile):
-        out = StringIO.StringIO()
+        out = NativeStringIO()
 
         # Patch before creating the pstats, because pstats binds self.stream to
         # sys.stdout early in 2.5 and newer.
@@ -998,91 +996,6 @@ class AppProfilingTests(unittest.TestCase):
         test_profilePrintStatsError.skip = "profile module not available"
 
 
-    def test_hotshot(self):
-        """
-        L{app.HotshotRunner.run} should call the C{run} method of the reactor
-        and save profile data in the specified file.
-        """
-        config = twistd.ServerOptions()
-        config["profile"] = self.mktemp()
-        config["profiler"] = "hotshot"
-        profiler = app.AppProfiler(config)
-        reactor = DummyReactor()
-
-        profiler.run(reactor)
-
-        self.assertTrue(reactor.called)
-        data = file(config["profile"]).read()
-        self.assertIn("run", data)
-        self.assertIn("function calls", data)
-
-    if hotshot is None:
-        test_hotshot.skip = "hotshot module not available"
-
-
-    def test_hotshotSaveStats(self):
-        """
-        With the C{savestats} option specified, L{app.HotshotRunner.run} should
-        save the raw stats object instead of a summary output.
-        """
-        config = twistd.ServerOptions()
-        config["profile"] = self.mktemp()
-        config["profiler"] = "hotshot"
-        config["savestats"] = True
-        profiler = app.AppProfiler(config)
-        reactor = DummyReactor()
-
-        profiler.run(reactor)
-
-        self.assertTrue(reactor.called)
-        self._testStats(hotshot.stats.load, config['profile'])
-
-    if hotshot is None:
-        test_hotshotSaveStats.skip = "hotshot module not available"
-
-
-    def test_withoutHotshot(self):
-        """
-        When the C{hotshot} module is not present, L{app.HotshotRunner.run}
-        should raise a C{SystemExit} exception and log the C{ImportError}.
-        """
-        savedModules = sys.modules.copy()
-        sys.modules["hotshot"] = None
-
-        config = twistd.ServerOptions()
-        config["profiler"] = "hotshot"
-        profiler = app.AppProfiler(config)
-        try:
-            self.assertRaises(SystemExit, profiler.run, None)
-        finally:
-            sys.modules.clear()
-            sys.modules.update(savedModules)
-
-
-    def test_hotshotPrintStatsError(self):
-        """
-        When an error happens while printing the stats, C{sys.stdout}
-        should be restored to its initial value.
-        """
-        class ErroneousStats(pstats.Stats):
-            def print_stats(self):
-                raise RuntimeError("Boom")
-        self.patch(pstats, "Stats", ErroneousStats)
-
-        config = twistd.ServerOptions()
-        config["profile"] = self.mktemp()
-        config["profiler"] = "hotshot"
-        profiler = app.AppProfiler(config)
-        reactor = DummyReactor()
-
-        oldStdout = sys.stdout
-        self.assertRaises(RuntimeError, profiler.run, reactor)
-        self.assertIdentical(sys.stdout, oldStdout)
-
-    if hotshot is None:
-        test_hotshotPrintStatsError.skip = "hotshot module not available"
-
-
     def test_cProfile(self):
         """
         L{app.CProfileRunner.run} should call the C{run} method of the
@@ -1097,7 +1010,8 @@ class AppProfilingTests(unittest.TestCase):
         profiler.run(reactor)
 
         self.assertTrue(reactor.called)
-        data = file(config["profile"]).read()
+        with open(config["profile"]) as f:
+            data = f.read()
         self.assertIn("run", data)
         self.assertIn("function calls", data)
 
@@ -1161,10 +1075,10 @@ class AppProfilingTests(unittest.TestCase):
 
     def test_defaultProfiler(self):
         """
-        L{app.Profiler} defaults to the hotshot profiler if not specified.
+        L{app.Profiler} defaults to the cprofile profiler if not specified.
         """
         profiler = app.AppProfiler({})
-        self.assertEqual(profiler.profiler, "hotshot")
+        self.assertEqual(profiler.profiler, "cprofile")
 
 
     def test_profilerNameCaseInsentive(self):
@@ -1172,8 +1086,8 @@ class AppProfilingTests(unittest.TestCase):
         The case of the profiler name passed to L{app.AppProfiler} is not
         relevant.
         """
-        profiler = app.AppProfiler({"profiler": "HotShot"})
-        self.assertEqual(profiler.profiler, "hotshot")
+        profiler = app.AppProfiler({"profiler": "CprOfile"})
+        self.assertEqual(profiler.profiler, "cprofile")
 
 
 
@@ -1577,7 +1491,7 @@ class DaemonizeTests(unittest.TestCase):
             self.runner.postApplication()
         self.assertEqual(
             self.mockos.actions,
-            [('chdir', '.'), ('umask', 077), ('fork', True), 'setsid',
+            [('chdir', '.'), ('umask', 0o077), ('fork', True), 'setsid',
              ('fork', True), ('write', -2, '0'), ('unlink', 'twistd.pid')])
         self.assertEqual(self.mockos.closed, [-3, -2])
 
@@ -1593,7 +1507,7 @@ class DaemonizeTests(unittest.TestCase):
             self.assertRaises(SystemError, self.runner.postApplication)
         self.assertEqual(
             self.mockos.actions,
-            [('chdir', '.'), ('umask', 077), ('fork', True),
+            [('chdir', '.'), ('umask', 0o077), ('fork', True),
              ('read', -1, 100), ('exit', 0), ('unlink', 'twistd.pid')])
         self.assertEqual(self.mockos.closed, [-1])
 
@@ -1615,7 +1529,7 @@ class DaemonizeTests(unittest.TestCase):
             self.runner.postApplication()
         self.assertEqual(
             self.mockos.actions,
-            [('chdir', '.'), ('umask', 077), ('fork', True), 'setsid',
+            [('chdir', '.'), ('umask', 0o077), ('fork', True), 'setsid',
              ('fork', True), ('unlink', 'twistd.pid')])
         self.assertEqual(self.mockos.closed, [-3, -2])
         self.assertEqual([(-2, '0'), (-2, '0')], written)
@@ -1640,7 +1554,7 @@ class DaemonizeTests(unittest.TestCase):
             self.assertRaises(SystemError, self.runner.postApplication)
         self.assertEqual(
             self.mockos.actions,
-            [('chdir', '.'), ('umask', 077), ('fork', True),
+            [('chdir', '.'), ('umask', 0o077), ('fork', True),
              ('exit', 0), ('unlink', 'twistd.pid')])
         self.assertEqual(self.mockos.closed, [-1])
         self.assertEqual([(-1, 100), (-1, 100)], read)
@@ -1664,7 +1578,7 @@ class DaemonizeTests(unittest.TestCase):
             self.assertRaises(RuntimeError, self.runner.postApplication)
         self.assertEqual(
             self.mockos.actions,
-            [('chdir', '.'), ('umask', 077), ('fork', True), 'setsid',
+            [('chdir', '.'), ('umask', 0o077), ('fork', True), 'setsid',
              ('fork', True), ('write', -2, '1 Something is wrong'),
              ('unlink', 'twistd.pid')])
         self.assertEqual(self.mockos.closed, [-3, -2])
@@ -1678,7 +1592,7 @@ class DaemonizeTests(unittest.TestCase):
         """
         self.mockos.child = False
         self.mockos.readData = "1: An identified error"
-        errorIO = StringIO.StringIO()
+        errorIO = NativeStringIO()
         self.patch(sys, '__stderr__', errorIO)
         with AlternateReactor(FakeDaemonizingReactor()):
             self.assertRaises(SystemError, self.runner.postApplication)
@@ -1688,7 +1602,7 @@ class DaemonizeTests(unittest.TestCase):
             "Please look at log file for more information.\n")
         self.assertEqual(
             self.mockos.actions,
-            [('chdir', '.'), ('umask', 077), ('fork', True),
+            [('chdir', '.'), ('umask', 0o077), ('fork', True),
              ('read', -1, 100), ('exit', 1), ('unlink', 'twistd.pid')])
         self.assertEqual(self.mockos.closed, [-1])
 
@@ -1711,7 +1625,7 @@ class DaemonizeTests(unittest.TestCase):
             self.assertRaises(RuntimeError, self.runner.postApplication)
         self.assertEqual(
             self.mockos.actions,
-            [('chdir', '.'), ('umask', 077), ('fork', True), 'setsid',
+            [('chdir', '.'), ('umask', 0o077), ('fork', True), 'setsid',
              ('fork', True), ('write', -2, '1 ' + 'x' * 98),
              ('unlink', 'twistd.pid')])
         self.assertEqual(self.mockos.closed, [-3, -2])
