@@ -1,7 +1,8 @@
 # Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-import StringIO
+from __future__ import absolute_import, division
+
 import gc
 import re
 import sys
@@ -9,16 +10,19 @@ import textwrap
 import types
 
 from twisted.trial import unittest
-from twisted.trial.runner import (
-    TrialRunner, TestSuite, DestructiveTestSuite, TestLoader)
-from twisted.trial._dist.disttrial import DistTrialRunner
+from twisted.trial.runner import TrialRunner, TestSuite, DestructiveTestSuite
+from twisted.trial.runner import TestLoader
 from twisted.scripts import trial
 from twisted.python import util
 from twisted.python.usage import UsageError
 from twisted.python.filepath import FilePath
+from twisted.python.compat import NativeStringIO, _PY3
 
 from twisted.trial.test.test_loader import testNames
 
+if not _PY3:
+    from twisted.trial._dist.disttrial import DistTrialRunner
+
 pyunit = __import__('unittest')
 
 
@@ -62,7 +66,7 @@ class ForceGarbageCollectionTests(unittest.SynchronousTestCase):
         Return a L{TrialRunner} object that is safe to use in tests.
         """
         runner = trial._makeRunner(self.config)
-        runner.stream = StringIO.StringIO()
+        runner.stream = NativeStringIO()
         return runner
 
 
@@ -193,14 +197,14 @@ class TestModuleTests(unittest.SynchronousTestCase):
         self.config.opt_testmodule(sibpath('scripttest.py'))
         self.assertSuitesEqual(trial._getSuite(self.config),
                                ['twisted.trial.test.test_log',
-                                'twisted.trial.test.test_class'])
+                                'twisted.trial.test.test_runner'])
 
     def test_testmoduleOnNonexistentFile(self):
         """
         Check that --testmodule displays a meaningful error message when
         passed a non-existent filename.
         """
-        buffy = StringIO.StringIO()
+        buffy = NativeStringIO()
         stderr, sys.stderr = sys.stderr, buffy
         filename = 'test_thisbetternoteverexist.py'
         try:
@@ -224,7 +228,7 @@ class TestModuleTests(unittest.SynchronousTestCase):
         Check that --testmodule does *not* support module names as arguments
         and that it displays a meaningful error message.
         """
-        buffy = StringIO.StringIO()
+        buffy = NativeStringIO()
         stderr, sys.stderr = sys.stderr, buffy
         moduleName = 'twisted.trial.test.test_script'
         try:
@@ -292,7 +296,7 @@ class TestModuleTests(unittest.SynchronousTestCase):
         localVars = trial.loadLocalVariables(sibpath('scripttest.py'))
         self.assertEqual(
             {'test-case-name': ('twisted.trial.test.test_log,'
-                                'twisted.trial.test.test_class')},
+                                'twisted.trial.test.test_runner')},
             localVars)
 
     def test_getTestModules(self):
@@ -307,15 +311,15 @@ class TestModuleTests(unittest.SynchronousTestCase):
         modules = trial.getTestModules(sibpath('scripttest.py'))
         self.assertEqual(set(modules),
                              set(['twisted.trial.test.test_log',
-                                  'twisted.trial.test.test_class']))
+                                  'twisted.trial.test.test_runner']))
 
     def test_looksLikeTestModule(self):
         for filename in ['test_script.py', 'twisted/trial/test/test_script.py']:
-            self.failUnless(trial.isTestFile(filename),
+            self.assertTrue(trial.isTestFile(filename),
                             "%r should be a test file" % (filename,))
         for filename in ['twisted/trial/test/moduletest.py',
                          sibpath('scripttest.py'), sibpath('test_foo.bat')]:
-            self.failIf(trial.isTestFile(filename),
+            self.assertFalse(trial.isTestFile(filename),
                         "%r should *not* be a test file" % (filename,))
 
 
@@ -571,6 +575,8 @@ class MakeRunnerTests(unittest.TestCase):
         self.assertEqual(4, runner._workerNumber)
         self.assertEqual(["--force-gc"], runner._workerArguments)
 
+    if _PY3:
+        test_jobs.skip = "DistTrialRunner is not yet ported to Python 3"
 
     def test_dryRunWithJobs(self):
         """
@@ -656,7 +662,7 @@ class TestArgumentOrderTests(unittest.TestCase):
         tests = [
             "twisted.trial.test.test_tests",
             "twisted.trial.test.test_assertions",
-            "twisted.trial.test.test_deferreds",
+            "twisted.trial.test.test_deferred",
             ]
         self.config.parseOptions(tests)
 
@@ -798,25 +804,25 @@ class OrderTests(unittest.TestCase):
         --order=toptobottom detects the source line of methods from modules
         whose source file is missing.
         """
-        tempdir = self.mktemp().encode('utf-8')
-        package = FilePath(tempdir).child(b'twisted_toptobottom_temp')
+        tempdir = self.mktemp()
+        package = FilePath(tempdir).child('twisted_toptobottom_temp')
         package.makedirs()
-        package.child(b'__init__.py').setContent(b'')
-        package.child(b'test_missing.py').setContent(textwrap.dedent(b'''
+        package.child('__init__.py').setContent(b'')
+        package.child('test_missing.py').setContent(textwrap.dedent('''
         from twisted.trial.unittest import TestCase
         class TestMissing(TestCase):
             def test_second(self): pass
             def test_third(self): pass
             def test_fourth(self): pass
             def test_first(self): pass
-        '''))
-        pathEntry = package.parent().path.decode('utf-8')
+        ''').encode('utf8'))
+        pathEntry = package.parent().path
         sys.path.insert(0, pathEntry)
         self.addCleanup(sys.path.remove, pathEntry)
         from twisted_toptobottom_temp import test_missing
         self.addCleanup(sys.modules.pop, 'twisted_toptobottom_temp')
         self.addCleanup(sys.modules.pop, test_missing.__name__)
-        package.child(b'test_missing.py').remove()
+        package.child('test_missing.py').remove()
 
         self.config.parseOptions([
             "--order", "toptobottom", "twisted.trial.test.ordertests"])
@@ -849,7 +855,7 @@ class HelpOrderTests(unittest.TestCase):
         """
         --help-orders prints each of the available orders and then exits.
         """
-        self.patch(sys, "stdout", StringIO.StringIO())
+        self.patch(sys, "stdout", NativeStringIO())
 
         exc = self.assertRaises(
             SystemExit, trial.Options().parseOptions, ["--help-orders"])
