@@ -5,6 +5,8 @@
 Test HTTP support.
 """
 
+from __future__ import absolute_import, division
+
 import random, cgi, base64
 
 try:
@@ -702,6 +704,19 @@ class ParsingTests(unittest.TestCase):
         return channel
 
 
+    def test_invalidNonAsciiMethod(self):
+        """
+        When client sends invalid HTTP method containing
+        non-ascii characters HTTP 400 'Bad Request' status will be returned.
+        """
+        badRequestLine = b"GE\xc2\xa9 / HTTP/1.1\r\n\r\n"
+        channel = self.runRequest(badRequestLine, http.Request, 0)
+        self.assertEqual(
+            channel.transport.value(),
+            b"HTTP/1.1 400 Bad Request\r\n\r\n")
+        self.assertTrue(channel.transport.disconnecting)
+
+
     def test_basicAuth(self):
         """
         L{HTTPChannel} provides username and password information supplied in
@@ -1019,7 +1034,11 @@ Content-Type: application/x-www-form-urlencoded
         self.assertEqual(content, [networkString(query)])
 
 
-    def testMissingContentDisposition(self):
+    def test_missingContentDisposition(self):
+        """
+        If the C{Content-Disposition} header is missing, the request is denied
+        as a bad request.
+        """
         req = b'''\
 POST / HTTP/1.0
 Content-Type: multipart/form-data; boundary=AaB03x
@@ -1032,11 +1051,71 @@ Content-Transfer-Encoding: quoted-printable
 abasdfg
 --AaB03x--
 '''
-        self.runRequest(req, http.Request, success=False)
+        channel = self.runRequest(req, http.Request, success=False)
+        self.assertEqual(
+            channel.transport.value(),
+            b"HTTP/1.1 400 Bad Request\r\n\r\n")
+
     if _PY3:
-        testMissingContentDisposition.skip = (
-            "Cannot parse multipart/form-data on Python 3.  "
-            "See http://bugs.python.org/issue12411 and #5511.")
+        test_missingContentDisposition.skip = (
+            "cgi.parse_multipart is much more error-tolerant on Python 3.")
+
+
+    def test_multipartProcessingFailure(self):
+        """
+        When the multipart processing fails the client gets a 400 Bad Request.
+        """
+        # The parsing failure is simulated by having a Content-Length that
+        # doesn't fit in a ssize_t.
+        req = b'''\
+POST / HTTP/1.0
+Content-Type: multipart/form-data; boundary=AaB03x
+Content-Length: 103
+
+--AaB03x
+Content-Type: text/plain
+Content-Length: 999999999999999999999999999999999999999999999999999999999999999
+Content-Transfer-Encoding: quoted-printable
+
+abasdfg
+--AaB03x--
+'''
+        channel = self.runRequest(req, http.Request, success=False)
+        self.assertEqual(
+            channel.transport.value(),
+            b"HTTP/1.1 400 Bad Request\r\n\r\n")
+
+
+    def test_multipartFormData(self):
+        """
+        If the request has a Content-Type of C{multipart/form-data}, and the
+        form data is parseable, the form arguments will be added to the
+        request's args.
+        """
+        processed = []
+        class MyRequest(http.Request):
+            def process(self):
+                processed.append(self)
+                self.write(b"done")
+                self.finish()
+        req = b'''\
+POST / HTTP/1.0
+Content-Type: multipart/form-data; boundary=AaB03x
+Content-Length: 149
+
+--AaB03x
+Content-Type: text/plain
+Content-Disposition: form-data; name="text"
+Content-Transfer-Encoding: quoted-printable
+
+abasdfg
+--AaB03x--
+'''
+        channel = self.runRequest(req, MyRequest, success=False)
+        self.assertEqual(channel.transport.value(),
+                         b"HTTP/1.0 200 OK\r\n\r\ndone")
+        self.assertEqual(len(processed), 1)
+        self.assertEqual(processed[0].args, {b"text": [b"abasdfg"]})
 
 
     def test_chunkedEncoding(self):
@@ -1271,22 +1350,6 @@ class RequestTests(unittest.TestCase, ResponseTestMixin):
         self.assertEqual(getattr(req, oldName), {})
 
 
-    def test_received_headers(self):
-        """
-        L{Request.received_headers} is a backwards compatible API which
-        accesses and allows mutation of the state at L{Request.requestHeaders}.
-        """
-        self._compatHeadersTest('received_headers', 'requestHeaders')
-
-
-    def test_headers(self):
-        """
-        L{Request.headers} is a backwards compatible API which accesses and
-        allows mutation of the state at L{Request.responseHeaders}.
-        """
-        self._compatHeadersTest('headers', 'responseHeaders')
-
-
     def test_getHeader(self):
         """
         L{http.Request.getHeader} returns the value of the named request
@@ -1366,13 +1429,24 @@ class RequestTests(unittest.TestCase, ResponseTestMixin):
         """
         channel = DummyChannel()
         req = http.Request(channel, False)
-        req.setResponseCode(202, "happily accepted")
+        req.setResponseCode(202, b"happily accepted")
         req.write(b'')
         self.assertEqual(
             channel.transport.written.getvalue().splitlines()[0],
             b'(no clientproto yet) 202 happily accepted')
 
 
+    def test_setResponseCodeAndMessageNotBytes(self):
+        """
+        L{http.Request.setResponseCode} accepts C{bytes} for the message
+        parameter and raises L{TypeError} if passed anything else.
+        """
+        channel = DummyChannel()
+        req = http.Request(channel, False)
+        self.assertRaises(TypeError, req.setResponseCode,
+                          202, u"not happily accepted")
+
+
     def test_setResponseCodeAcceptsIntegers(self):
         """
         L{http.Request.setResponseCode} accepts C{int} for the code parameter
@@ -1452,6 +1526,31 @@ class RequestTests(unittest.TestCase, ResponseTestMixin):
         self.assertEqual(req.responseHeaders.getRawHeaders(b"test"), [b"lemur"])
 
 
+    def test_addCookieWithMinimumArguments(self):
+        """
+        Add a Set-Cookie header with just name and value to the response.
+        """
+        req = http.Request(DummyChannel(), False)
+        req.addCookie("foo", "bar")
+        self.assertEqual(req.cookies[0], "foo=bar")
+
+
+    def test_addCookieWithAllArguments(self):
+        """
+        Add a Set-Cookie header with name and value and all the supported
+        options to the response.
+        """
+        req = http.Request(DummyChannel(), False)
+        req.addCookie(
+            "foo", "bar", expires="Fri, 31 Dec 9999 23:59:59 GMT",
+            domain=".example.com", path="/", max_age="31536000",
+            comment="test", secure=True, httpOnly=True)
+        self.assertEqual(req.cookies[0],
+                         "foo=bar; Expires=Fri, 31 Dec 9999 23:59:59 GMT; "
+                         "Domain=.example.com; Path=/; Max-Age=31536000; "
+                         "Comment=test; Secure; HttpOnly")
+
+
     def test_firstWrite(self):
         """
         For an HTTP 1.0 request, L{http.Request.write} sends an HTTP 1.0
@@ -2197,82 +2296,6 @@ class DeprecatedRequestAttributesTests(unittest.TestCase):
     """
     Tests for deprecated attributes of L{twisted.web.http.Request}.
     """
-    def test_readHeaders(self):
-        """
-        Reading from the C{headers} attribute is deprecated in favor of use of
-        the C{responseHeaders} attribute.
-        """
-        request = http.Request(DummyChannel(), True)
-        request.headers
-        warnings = self.flushWarnings(
-            offendingFunctions=[self.test_readHeaders])
-
-        self.assertEqual({
-                "category": DeprecationWarning,
-                "message": (
-                    "twisted.web.http.Request.headers was deprecated in "
-                    "Twisted 13.2.0: Please use twisted.web.http.Request."
-                    "responseHeaders instead.")},
-                         sub(["category", "message"], warnings[0]))
-
-
-    def test_writeHeaders(self):
-        """
-        Writing to the C{headers} attribute is deprecated in favor of use of
-        the C{responseHeaders} attribute.
-        """
-        request = http.Request(DummyChannel(), True)
-        request.headers = {b"foo": b"bar"}
-        warnings = self.flushWarnings(
-            offendingFunctions=[self.test_writeHeaders])
-
-        self.assertEqual({
-                "category": DeprecationWarning,
-                "message": (
-                    "twisted.web.http.Request.headers was deprecated in "
-                    "Twisted 13.2.0: Please use twisted.web.http.Request."
-                    "responseHeaders instead.")},
-                         sub(["category", "message"], warnings[0]))
-
-
-    def test_readReceivedHeaders(self):
-        """
-        Reading from the C{received_headers} attribute is deprecated in favor
-        of use of the C{requestHeaders} attribute.
-        """
-        request = http.Request(DummyChannel(), True)
-        request.received_headers
-        warnings = self.flushWarnings(
-            offendingFunctions=[self.test_readReceivedHeaders])
-
-        self.assertEqual({
-                "category": DeprecationWarning,
-                "message": (
-                    "twisted.web.http.Request.received_headers was deprecated "
-                    "in Twisted 13.2.0: Please use twisted.web.http.Request."
-                    "requestHeaders instead.")},
-                         sub(["category", "message"], warnings[0]))
-
-
-    def test_writeReceivedHeaders(self):
-        """
-        Writing to the C{received_headers} attribute is deprecated in favor of use of
-        the C{requestHeaders} attribute.
-        """
-        request = http.Request(DummyChannel(), True)
-        request.received_headers = {b"foo": b"bar"}
-        warnings = self.flushWarnings(
-            offendingFunctions=[self.test_writeReceivedHeaders])
-
-        self.assertEqual({
-                "category": DeprecationWarning,
-                "message": (
-                    "twisted.web.http.Request.received_headers was deprecated "
-                    "in Twisted 13.2.0: Please use twisted.web.http.Request."
-                    "requestHeaders instead.")},
-                         sub(["category", "message"], warnings[0]))
-
-
     def test_getClient(self):
         """
         L{Request.getClient} is deprecated in favor of resolving the hostname
