@@ -11,6 +11,7 @@ import zlib
 from zope.interface import implementer
 from zope.interface.verify import verifyObject
 
+from twisted.python import reflect, failure
 from twisted.python.compat import _PY3
 from twisted.python.filepath import FilePath
 from twisted.trial import unittest
@@ -491,6 +492,66 @@ class RequestTests(unittest.TestCase):
         self.assertEqual(request.prePathURL(), b'http://example.com/foo%2Fbar')
 
 
+    def test_processingFailedNoTraceback(self):
+        """
+        L{Request.processingFailed} when the site has C{displayTracebacks} set
+        to C{False} does not write out the failure, but give a generic error
+        message.
+        """
+        d = DummyChannel()
+        request = server.Request(d, 1)
+        request.site = server.Site(resource.Resource())
+        request.site.displayTracebacks = False
+        fail = failure.Failure(Exception("Oh no!"))
+        request.processingFailed(fail)
+
+        self.assertNotIn(b"Oh no!", request.transport.getvalue())
+        self.assertIn(b"Processing Failed", request.transport.getvalue())
+
+        # Since we didn't "handle" the exception, flush it to prevent a test
+        # failure
+        self.assertEqual(1, len(self.flushLoggedErrors()))
+
+
+    def test_processingFailedDisplayTraceback(self):
+        """
+        L{Request.processingFailed} when the site has C{displayTracebacks} set
+        to C{True} writes out the failure.
+        """
+        d = DummyChannel()
+        request = server.Request(d, 1)
+        request.site = server.Site(resource.Resource())
+        request.site.displayTracebacks = True
+        fail = failure.Failure(Exception("Oh no!"))
+        request.processingFailed(fail)
+
+        self.assertIn(b"Oh no!", request.transport.getvalue())
+
+        # Since we didn't "handle" the exception, flush it to prevent a test
+        # failure
+        self.assertEqual(1, len(self.flushLoggedErrors()))
+
+
+    def test_processingFailedDisplayTracebackHandlesUnicode(self):
+        """
+        L{Request.processingFailed} when the site has C{displayTracebacks} set
+        to C{True} writes out the failure, making UTF-8 items into HTML
+        entities.
+        """
+        d = DummyChannel()
+        request = server.Request(d, 1)
+        request.site = server.Site(resource.Resource())
+        request.site.displayTracebacks = True
+        fail = failure.Failure(Exception(u"\u2603"))
+        request.processingFailed(fail)
+
+        self.assertIn(b"&#9731;", request.transport.getvalue())
+
+        # Since we didn't "handle" the exception, flush it to prevent a test
+        # failure
+        self.assertEqual(1, len(self.flushLoggedErrors()))
+
+
 
 class GzipEncoderTests(unittest.TestCase):
 
@@ -728,6 +789,19 @@ class NewRenderTests(unittest.TestCase):
         req.requestReceived(b'hlalauguG', b'/newrender', b'HTTP/1.0')
         self.assertEqual(req.code, 501)
 
+    def test_notAllowedMethod(self):
+        """
+        When trying to invoke a method not in the allowed method list, we get
+        a response saying it is not allowed.
+        """
+        req = self._getReq()
+        req.requestReceived(b'POST', b'/newrender', b'HTTP/1.0')
+        self.assertEqual(req.code, 405)
+        self.assertTrue(req.responseHeaders.hasHeader(b"allow"))
+        raw_header = req.responseHeaders.getRawHeaders(b'allow')[0]
+        allowed = sorted([h.strip() for h in raw_header.split(b",")])
+        self.assertEqual([b'GET', b'HEAD', b'HEH'], allowed)
+
     def testImplicitHead(self):
         req = self._getReq()
         req.requestReceived(b'HEAD', b'/newrender', b'HTTP/1.0')
@@ -748,6 +822,42 @@ class NewRenderTests(unittest.TestCase):
         self.assertEqual(body, b'')
 
 
+    def test_noBytesResult(self):
+        """
+        When implemented C{render} method does not return bytes an internal
+        server error is returned.
+        """
+        class RiggedRepr(object):
+            def __repr__(self):
+                return 'my>repr'
+
+        result = RiggedRepr()
+        no_bytes_resource = resource.Resource()
+        no_bytes_resource.render = lambda request: result
+        request = self._getReq(no_bytes_resource)
+
+        request.requestReceived(b"GET", b"/newrender", b"HTTP/1.0")
+
+        headers, body = request.transport.getvalue().split(b'\r\n\r\n')
+        self.assertEqual(request.code, 500)
+        expected = [
+            '',
+            '<html>',
+            '  <head><title>500 - Request did not return bytes</title></head>',
+            '  <body>',
+            '    <h1>Request did not return bytes</h1>',
+            '    <p>Request: <pre>&lt;%s&gt;</pre><br />'
+                'Resource: <pre>&lt;%s&gt;</pre><br />'
+                'Value: <pre>my&gt;repr</pre></p>' % (
+                    reflect.safe_repr(request)[1:-1],
+                    reflect.safe_repr(no_bytes_resource)[1:-1],
+                    ),
+            '  </body>',
+            '</html>',
+            '']
+        self.assertEqual('\n'.join(expected).encode('ascii'), body)
+
+
 
 class GettableResource(resource.Resource):
     """
@@ -992,8 +1102,8 @@ class CombinedLogFormatterTests(unittest.TestCase):
         request.client = IPv4Address("TCP", b"evil x-forwarded-for \x80", 12345)
         request.method = b"POS\x81"
         request.protocol = b"HTTP/1.\x82"
-        request.headers[b"referer"] = b"evil \x83"
-        request.headers[b"user-agent"] = b"evil \x84"
+        request.requestHeaders.addRawHeader(b"referer", b"evil \x83")
+        request.requestHeaders.addRawHeader(b"user-agent", b"evil \x84")
 
         line = http.combinedLogFormatter(timestamp, request)
         self.assertEqual(
@@ -1136,7 +1246,8 @@ class LogEscapingTests(unittest.TestCase):
         """
         self.site._logDateTime = "[%02d/%3s/%4d:%02d:%02d:%02d +0000]" % (
             25, 'Oct', 2004, 12, 31, 59)
-        self.request.headers[b'referer'] = (
+        self.request.requestHeaders.addRawHeader(
+            b'referer',
             b'http://malicious" ".website.invalid')
         self.assertLogs(
             b'"1.2.3.4" - - [25/Oct/2004:12:31:59 +0000] '
@@ -1151,7 +1262,8 @@ class LogEscapingTests(unittest.TestCase):
         """
         self.site._logDateTime = "[%02d/%3s/%4d:%02d:%02d:%02d +0000]" % (
             25, 'Oct', 2004, 12, 31, 59)
-        self.request.headers[b'user-agent'] = b'Malicious Web" Evil'
+        self.request.requestHeaders.addRawHeader(b'user-agent', 
+                                                 b'Malicious Web" Evil')
         self.assertLogs(
             b'"1.2.3.4" - - [25/Oct/2004:12:31:59 +0000] '
             b'"GET /dummy HTTP/1.0" 123 - "-" "Malicious Web\\" Evil"\n')
